import {get} from "lodash";

import createBootstrapStyleSheet from "./create-bootstrap-stylesheet";

function TestRule(themeProps = {}) {
  return {fontSize: themeProps.myFontSize};
}
TestRule.className = "test";

const testThemeProps = {
  myFontSize: "20px",
};

describe("createStyleSheet(preserveClassNames:false)",()=>{
  const sheet = createBootstrapStyleSheet([TestRule],testThemeProps);
  it("should have no attached stylesheets",()=>{
    expect(document.styleSheets.length).toBe(0);
  });
  it("should add a stylesheet on attach",()=>{
    sheet.attach();
    expect(document.styleSheets.length).toBe(1);
  });
  it("should return a JSS stylesheet",()=>{
    expect(sheet).toHaveProperty("classes.test");
    expect(get(sheet,"classes.test")).not.toBe("test");
  });
  it("should remove the stylesheet on detach",()=>{
    sheet.detach();
    expect(document.styleSheets.length).toBe(0);
  });

});
describe("createStyleSheet()",()=>{

  const sheet = createBootstrapStyleSheet([TestRule],testThemeProps,{preserveClassNames: true});
  const el = document.createElement("div");
  document.body.appendChild(el);

  it("should return the unmodified classname",()=>{
    expect(get(sheet,"classes.test")).toBe("test");
  });
  it("should produce the expected css",()=>{
    expect(sheet.toString().replace(/\s+/g," ")).toBe(".test { font-size: 20px; } .root .test { font-size: 20px; }");
  });

  it("should have no attached stylesheets",()=>{
    expect(document.styleSheets.length).toBe(0);
  });

  it("should add a stylesheet on attach",()=>{
    sheet.attach();
    expect(document.styleSheets.length).toBe(1);
  });
  it("should not have any effect on classless div",()=>{
    expect(window.getComputedStyle(el).getPropertyValue("font-size")).toBe("");
  });
  it("should have an effect on div with test class",()=>{
    el.setAttribute("class","test");
    expect(window.getComputedStyle(el).getPropertyValue("font-size")).toBe("20px");
  });
  it("should remove the stylesheet on detach",()=>{
    sheet.detach();
    expect(document.styleSheets.length).toBe(0);
  });
  it("should have remove the styling on the test div once detached",()=>{
    expect(window.getComputedStyle(el).getPropertyValue("font-size")).not.toBe("20px");
  });

});
describe("createStyleSheet(namespace)",()=>{

  const sheet = createBootstrapStyleSheet([TestRule],testThemeProps,{namespace: "mytest",preserveClassNames: true});
  const outsideEl = document.createElement("span");
  outsideEl.setAttribute("class","test outside");
  const insideEl = document.createElement("span");
  insideEl.setAttribute("class","test inside");
  const container = document.createElement("div");
  container.setAttribute("class","mytest");
  container.appendChild(insideEl);
  document.body.appendChild(container);
  document.body.appendChild(outsideEl);

  it("should return the unmodified classname",()=>{
    expect(get(sheet,"classes.test")).toBe("test");
  });
  it("should produce the expected css",()=>{
    expect(sheet.toString().replace(/\s+/g," ")).toBe(".mytest .test { font-size: 20px; }");
  });

  it("should have no attached stylesheets",()=>{
    expect(document.styleSheets.length).toBe(0);
  });

  it("should add a stylesheet on attach",()=>{
    sheet.attach();
    expect(document.styleSheets.length).toBe(1);
  });
  it("should not have any effect on classless container",()=>{
    expect(window.getComputedStyle(outsideEl).getPropertyValue("font-size")).toBe("");
  });
  it("should have an effect on div with test class",()=>{
    container.setAttribute("class","mytest");
    expect(window.getComputedStyle(insideEl).getPropertyValue("font-size")).toBe("20px");
  });
  it("should remove the stylesheet on detach",()=>{
    sheet.detach();
    expect(document.styleSheets.length).toBe(0);
  });

});
