import {get} from "lodash";
import {createBootstrapStyleSheet} from "../index";
import {button,coloredButton,outlinedButton,themedButtons,largeButton,smallButton,blockButton} from "./buttons";
import {validProps,validateCSS} from "../lib/test-utils";


describe("buttons",()=>{
  describe("buttonRule",()=>{
    describe("default buttonRule",()=>{
      const defaultButtonStyles = button();
      it("should return some valid style objects",()=>{
        expect(typeof defaultButtonStyles).toBe("object");
        expect(validProps(defaultButtonStyles)).toBe(true);
        expect(defaultButtonStyles).toMatchSnapshot();
      });
    });

    describe("bootstrap",()=>{
      const sheet = createBootstrapStyleSheet([button],{},{preserveClassNames: true});
      const css = sheet.toString();

      it("should return a valid style object",()=>{
        expect(validProps(sheet.rules.raw)).toBe(true);
      });
      it("should return valid css",()=>{
        return validateCSS(css).then((valid)=>{
          expect(valid).toBe(true);
        });
      });
      it("should match css snapshot",()=>{
        expect(sheet.toString()).toMatchSnapshot();
      });

      it("should have a btn class",()=>{
        expect(get(sheet,"classes.btn")).toBe("btn");
      });
      it("should have the default fontSize",()=>{
        expect(sheet.getRule("btn").style["font-size"]).toBe("1rem");
      });
    });

    describe("bootstrap",()=>{
      const sheet = createBootstrapStyleSheet([button],{},{preserveClassNames: true});

      const css = sheet.toString();

      it("should return a valid style object",()=>{
        expect(validProps(sheet.rules.raw)).toBe(true);
      });
      it("should return valid css",()=>{
        return validateCSS(css).then((valid)=>{
          expect(valid).toBe(true);
        });
      });
      it("should match css snapshot",()=>{
        expect(sheet.toString()).toMatchSnapshot();
      });

      it("should have a btn class",()=>{
        expect(get(sheet,"classes.btn")).toBe("btn");
      });

      it("should have the default fontSize",()=>{
        expect(sheet.getRule("btn").style["font-size"]).toBe("1rem");
      });

    });

    describe("bootstrap (themed)",()=>{
      const sheet = createBootstrapStyleSheet([button],{font: {sizeBase: "2rem"}},{preserveClassNames: true});

      const css = sheet.toString();

      it("should return a valid style object",()=>{
        expect(validProps(sheet.rules.raw)).toBe(true);
      });
      it("should return valid css",()=>{
        return validateCSS(css).then((valid)=>{
          expect(valid).toBe(true);
        });
      });
      it("should match css snapshot",()=>{
        expect(sheet.toString()).toMatchSnapshot();
      });

      it("should have the themed fontSize",()=>{
        expect(sheet.getRule("btn").style["font-size"]).toBe("2rem");
      });

    });

  });

  describe("coloredButton",()=>{
    const blue = "#000099";
    const sheet = createBootstrapStyleSheet([coloredButton("blue",blue)],{},{preserveClassNames: true});
    const css = sheet.toString();

    it("should return a valid style object",()=>{
      expect(validProps(sheet.rules.raw)).toBe(true);
    });
    it("should return valid css",()=>{
      return validateCSS(css).then((valid)=>{
        expect(valid).toBe(true);
      });
    });
    it("should match css snapshot",()=>{
      expect(sheet.toString()).toMatchSnapshot();
    });
    it("should have the correct background color",()=>{
      expect(sheet.getRule("btn-blue").style["background-color"]).toBe(blue);
    });

  });

  describe("outlinedButton",()=>{
    const blue = "#000099";
    const sheet = createBootstrapStyleSheet([outlinedButton("blue",blue)],{},{preserveClassNames: true});
    const css = sheet.toString();

    it("should return a valid style object",()=>{
      expect(validProps(sheet.rules.raw)).toBe(true);
    });
    it("should return valid css",()=>{
      return validateCSS(css).then((valid)=>{
        expect(valid).toBe(true);
      });
    });
    it("should match css snapshot",()=>{
      expect(sheet.toString()).toMatchSnapshot();
    });

    it("should have the correct border color",()=>{
      expect(sheet.getRule("btn-outline-blue").style["border-color"]).toBe(blue);
    });


  });

  describe("themedButtons (defaults)",()=>{
    const sheet = createBootstrapStyleSheet([themedButtons],{},{preserveClassNames: true});
    const css = sheet.toString();

    it("should return a valid style object",()=>{
      expect(validProps(sheet.rules.raw)).toBe(true);
    });
    it("should return valid css",()=>{
      return validateCSS(css).then((valid)=>{
        expect(valid).toBe(true);
      });
    });
    it("should match css snapshot",()=>{
      expect(sheet.toString()).toMatchSnapshot();
    });

    it("should have the correct outline border color",()=>{
      expect(sheet.getRule("btn-outline-primary").style["border-color"]).toBe("#007bff");
    });

    it("should have the correct background color",()=>{
      expect(sheet.getRule("btn-primary").style["background-color"]).toBe("#007bff");
    });

  });

  describe("themedButtons (additional theme)",()=>{
    const foo = "#990000";
    const sheet = createBootstrapStyleSheet([themedButtons],{
      color: {
        themeColors: {
          foo,
        }
      }
    },{preserveClassNames: true});
    const css = sheet.toString();

    it("should return a valid style object",()=>{
      expect(validProps(sheet.rules.raw)).toBe(true);
    });
    it("should return valid css",()=>{
      return validateCSS(css).then((valid)=>{
        expect(valid).toBe(true);
      });
    });
    it("should match css snapshot",()=>{
      expect(sheet.toString()).toMatchSnapshot();
    });

    it("should have a btn-foo class",()=>{
      expect(sheet.classes).toHaveProperty("btn-foo");
    });

    it("should have a btn-primary class",()=>{
      expect(sheet.classes).toHaveProperty("btn-primary");
    });

    it("should have the correct outline border color",()=>{
      expect(sheet.getRule("btn-outline-foo").style["border-color"]).toBe(foo);
    });

    it("should have the correct background color",()=>{
      expect(sheet.getRule("btn-foo").style["background-color"]).toBe(foo);
    });

  });

  describe("themedButtons (replaced themes)",()=>{
    const foo = "#990000";
    const sheet = createBootstrapStyleSheet([themedButtons],{
      options: {
        themeColors: ["foo"],
      },
      color: {
        themeColors: {
          foo,
        }
      }
    },{preserveClassNames: true});
    const css = sheet.toString();

    it("should return a valid style object",()=>{
      expect(validProps(sheet.rules.raw)).toBe(true);
    });
    it("should return valid css",()=>{
      return validateCSS(css).then((valid)=>{
        expect(valid).toBe(true);
      });
    });
    it("should match css snapshot",()=>{
      expect(sheet.toString()).toMatchSnapshot();
    });

    it("should have a btn-foo class",()=>{
      expect(sheet.classes).toHaveProperty("btn-foo");
    });

    it("should not have a btn-primary class",()=>{
      expect(sheet.classes).not.toHaveProperty("btn-primary");
    });

    it("should have the correct outline border color",()=>{
      expect(sheet.getRule("btn-outline-foo").style["border-color"]).toBe(foo);
    });

    it("should have the correct background color",()=>{
      expect(sheet.getRule("btn-foo").style["background-color"]).toBe(foo);
    });

  });

  describe("largeButton",()=>{

    const sheet = createBootstrapStyleSheet([largeButton],{},{preserveClassNames: true});
    const css = sheet.toString();

    it("should return a valid style object",()=>{
      expect(validProps(sheet.rules.raw)).toBe(true);
    });
    it("should return valid css",()=>{
      return validateCSS(css).then((valid)=>{
        expect(valid).toBe(true);
      });
    });
    it("should match css snapshot",()=>{
      expect(sheet.toString()).toMatchSnapshot();
    });

    it("should have the correct font size",()=>{
      expect(sheet.getRule("btn-lg").style["font-size"]).toBe("1.25rem");
    });


  });

  describe("smallButton",()=>{

    const sheet = createBootstrapStyleSheet([smallButton],{},{preserveClassNames: true});
    const css = sheet.toString();

    it("should return a valid style object",()=>{
      expect(validProps(sheet.rules.raw)).toBe(true);
    });
    it("should return valid css",()=>{
      return validateCSS(css).then((valid)=>{
        expect(valid).toBe(true);
      });
    });
    it("should match css snapshot",()=>{
      expect(sheet.toString()).toMatchSnapshot();
    });

    it("should have the correct font size",()=>{
      expect(sheet.getRule("btn-sm").style["font-size"]).toBe(".875rem");
    });


  });

  describe("blockButton",()=>{

    const sheet = createBootstrapStyleSheet([blockButton],{},{preserveClassNames: true});
    const css = sheet.toString();

    it("should return a valid style object",()=>{
      expect(validProps(sheet.rules.raw)).toBe(true);
    });
    it("should return valid css",()=>{
      return validateCSS(css).then((valid)=>{
        expect(valid).toBe(true);
      });
    });
    it("should match css snapshot",()=>{
      expect(sheet.toString()).toMatchSnapshot();
    });

    it("should have the correct font size",()=>{
      expect(sheet.getRule("btn-block").style.width).toBe("100%");
    });


  });

});
