import {range,get} from "lodash";
import {multiply,add,lighten,darken,rgba,fuse} from "./lib/utils";
// Variables
//
// Variables should follow the `$component-state-property-size` formula for
// consistent naming. Ex: $nav-link-disabled-color and $modal-content-box-shadow-xs.


//
// Color system
//

// stylelint-disable
export const color = {
  defaults: {
    white: "#fff",
    gray100: "#f8f9fa",
    gray200: "#e9ecef",
    gray300: "#dee2e6",
    gray400: "#ced4da",
    gray500: "#adb5bd",
    gray600: "#868e96",
    gray700: "#495057",
    gray800: "#343a40",
    gray900: "#212529",
    black: "#000",
    blue: "#007bff",
    indigo: "#6610f2",
    purple: "#6f42c1",
    pink: "#e83e8c",
    red: "#dc3545",
    orange: "#fd7e14",
    yellow: "#ffc107",
    green: "#28a745",
    teal: "#20c997",
    cyan: "#17a2b8",
    // Set a specific jump point for requesting color jumps
    themeColorInterval: "8%",
  },
  normalize: (props)=>{

    const grays = {};
    range(100,10000,100).forEach((i)=>{
      grays[`${i}`] = props[`gray${i}`];
    });

    const colors = {
      blue: props.blue,
      indigo: props.indigo,
      purple: props.purple,
      pink: props.pink,
      red: props.red,
      orange: props.orange,
      yellow: props.yellow,
      green: props.green,
      teal: props.teal,
      cyan: props.cyan,
      white: props.white,
      gray: props.gray600,
      grayDark: props.gray800,
    };

    const themeColors = {

      primary: props.blue,
      secondary: props.gray600,
      success: props.green,
      info: props.cyan,
      warning: props.yellow,
      danger: props.red,
      light: props.gray100,
      dark: props.gray800,

    };

    return fuse({grays,colors,themeColors},props);

  }

};


// Options
//
// Quickly modify global styling by enabling or disabling optional features.
export const options = {
  defaults: {
    enableCaret: true,
    enableRounded: true,
    enableShadows: true,
    enableGradients: false,
    enableTransitions: true,
    enableHoverMediaQuery: false,
    enableGridClasses: true,
    enablePrintStyles: true,
  },
  normalize: (props,{getProps})=>{
    const {colorProps} = getProps({color});
    return fuse({
      themeColors: Object.keys(colorProps.themeColors),
    },props);
  }
};


// Spacing
//
// Control the default styling of most Bootstrap elements by modifying these
// variables. Mostly focused on spacing.
// You can add more entries to the $spacers map, should you need more variation.

export const spacing = {
  defaults: {
    spacer: "1 rem",
  },
  normalize: (props)=>{
    const spacer = props.spacer;
    const spacers = {
      0: 0,
      1: multiply(spacer,0.25),
      2: multiply(spacer,0.5),
      3: spacer,
      5: multiply(spacer,3),
    };
    return fuse({spacers},props);
  },
};



// This variable affects the `.h-*` and `.w-*` classes.
export const sizes = {
  defaults: {
    25: "25%",
    50: "50%",
    75: "75%",
    100: "100%",
  }
};

// Body
//
// Settings for the `<body>` element.
export const body = {
  normalize: (props,{getProps})=>{
    const {colorProps} = getProps({color});
    return fuse({
      bg: colorProps.white,
      color: colorProps.gray900,
    },props);
  }
};

// Links
//
// Style anchor elements.

export const link = {
  defaults: {
    decoration: "none",
    hoverDecoration: "underline",
  },
  normalize: (props,{getProps})=>{
    const {colorProps} = getProps({color});
    const linkColor = colorProps.themeColors.primary;
    return fuse({
      color: linkColor,
      hoverColor: darken(linkColor,0.15),
    },props);
  }
};

// Paragraphs
//
// Style p element.

export const paragraph = {
  defaults: {
    marginBottom: "1rem",
  }
};


// Grid breakpoints
//
// Define the minimum dimensions at which your layout will change,
// adapting to different screen sizes, for use in media queries.

// $grid-breakpoints: (
//   xs: 0,
//   sm: 576px,
//   md: 768px,
//   lg: 992px,
//   xl: 1200px
// ) !default;
// @include _assert-ascending($grid-breakpoints, "$grid-breakpoints");
// @include _assert-starts-at-zero($grid-breakpoints);


// Grid containers
//
// Define the maximum width of `.container` for different screen sizes.

// $container-max-widths: (
//   sm: 540px,
//   md: 720px,
//   lg: 960px,
//   xl: 1140px
// ) !default;
// @include _assert-ascending($container-max-widths, "$container-max-widths");


// Grid columns
//
// Set the number of columns and specify the width of the gutters.

// $grid-columns: 12 !default;
// $grid-gutter-width: 30px !default;

// Components
//
// Define common padding and border radius sizes and more.

export const component = {
  normalize: (props,{getProps})=>{
    const {colorProps} = getProps({color});
    return fuse({
      lineHeightLarge: 1.5,
      lineHeightSmall: 1.5,
      borderWidth: "1px",
      borderColor: colorProps.gray200,
      borderRadius: ".25rem",
      borderRadiusLarge: ".3rem",
      borderRadiusSmall: ".2rem",
      activeColor: colorProps.white,
      activeBG: colorProps.themeColors.primary,
      caretWidth: ".3em",
      transitionBase: "all .2s ease-in-out",
      transitionFade: "opacity .15s linear",
      transitionCollapse: "height .35s ease",
    },props);
  },
};

// Fonts
//
// Font, line-height, and color for body text, headings, and more.
export const font = {

  defaults: {
    familySansSerif: '-apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, "Helvetica Neue", Arial, sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol"',
    familyMonospace: '"SFMono-Regular", Menlo, Monaco, Consolas, "Liberation Mono", "Courier New", monospace',
    weightNormal: "normal",
    headingsLineHeight: 1.1,
    sizeBase: "1rem", // Assumes the browser default, typically `16px`
    weightBold: "bold",
  },

  normalize(props,{getProps}) {

    const {spacingProps,colorProps,componentProps} = getProps({spacing,color,component});

    return fuse({

      familyBase: props.familySansSerif,
      sizeLarge: "1.25rem",
      sizeSmall: ".875rem",
      weightLight: 300,
      weightBase: props.weightNormal,

      lineHeightBase: 1.5,

      h1Size: "2.5rem",
      h2Size: "2rem",
      h3Size: "1.75rem",
      h4Size: "1.5rem",
      h5Size: "1.25rem",
      h6Size: "1rem",

      headingsMarginBottom: multiply(spacingProps.spacer,0.5),
      headingsFamily: "inherit",
      headingsWeight: 500,
      headingsColor: "inherit",

      display1Size: "6rem",
      display2Size: "5.5rem",
      display3Size: "4.5rem",
      display4Size: "3.5rem",

      display1Weight: 300,
      display2Weight: 300,
      display3Weight: 300,
      display4Weight: 300,
      displayLineHeight: props.headingsLineHeight,

      leadSize: "1.25rem",
      leadWeight: 300,

      smallSize: "80%",

      textMuted: colorProps.gray600,

      blockquoteSmallColor: colorProps.gray600,
      blockquoteSize: multiply(props.sizeBase,1.25),

      hrBorderColor: rgba(colorProps.black,0.1),
      hrBorderWidth: componentProps.borderWidth,

      markPadding: ".2em",

      dtWeight: props.weightBold,

      kbdBoxShadow: `inset 0 -.1rem 0 ${rgba(colorProps.black,0.25)}`,
      nestedKbdWeight: props.weightBold,

      listInlinePadding: "5px",

      markBG: "#fcf8e3",
    },props);


  }
};


// Tables
//
// Customizes the `.table` component with basic values, each used across all table variations.

export const table = {
  normalize(props,{getProps}) {
    const {colorProps,componentProps,bodyProps} = getProps({color,component,body});
    const hoverBG = get(props,"hoverBG",rgba(colorProps.black,0.075));

    return fuse({
      cellPadding: ".75rem",
      cellPaddingSmall: ".3rem",
      bg: "transparent",
      accentBG: rgba(colorProps.black,0.05),
      hoverBG,
      activeBG: hoverBG,
      borderWidth: componentProps.borderWidth,
      borderColor: colorProps.gray200,
      headBG: colorProps.gray200,
      headColor: colorProps.gray700,
      darkBG: colorProps.gray900,
      darkAccentBG: rgba(colorProps.white, 0.05),
      darkHoverBG: rgba(colorProps.white, 0.075),
      darkBorderColor: lighten(colorProps.gray900, 0.075),
      darkColor: bodyProps.bg,
    },props);
  },
};

// Buttons
//
// For each of Bootstrap's buttons, define text, background and border color.

export const button = {
  normalize(props,{getProps}) {

    const {fontProps,colorProps,componentProps} = getProps({font,color,component});

    return fuse({
      inputPaddingY: ".375rem",
      inputPaddingX: ".75rem",
      inputLineHeight: fontProps.lineHeightBase,
      inputPaddingYSmall: ".25rem",
      inputPaddingXSmall: ".5rem",
      inputLineHeightSmall: componentProps.lineHeightSmall,

      inputPaddingYLarge: ".5rem",
      inputPaddingXLarge: "1rem",
      inputLineHeightLarge: componentProps.lineHeightLarge,

      inputBorderWidth: get(props,"inputBorderWidth",componentProps.borderWidth),

      fontWeight: fontProps.weightNormal,
      boxShadow: `inset 0 1px 0 ${rgba(colorProps.white,0.15)}, 0 1px 1px ${rgba(colorProps.black,0.075)}`,
      focusBoxShadow: `0 0 0 3px ${rgba(colorProps.themeColors.primary, 0.25)}`,
      activeBoxShadow: `inset 0 3px 5px ${rgba(colorProps.black,0.125)}`,

      linkDisabledColor: colorProps.gray600,

      blockSpacingY: ".5rem" ,

      // Allows for customizing button radius independently from global border radius
      borderRadius: componentProps.borderRadius,
      borderRadiusLarge: componentProps.borderRadiusLarge,
      borderRadiusSmall: componentProps.borderRadiusSmall,

      transition: "background-color .15s ease-in-out, border-color .15s ease-in-out, box-shadow .15s ease-in-out",
    },props);
  }
};

// Forms

export const form = {



  normalize(props,{getProps}) {

    const {colorProps,componentProps,buttonProps,fontProps,bodyProps} = getProps({color,component,button,font,body});

    const inputBG = get(props,"inputBG",colorProps.white);
    const inputColor = get(props,"inputColor",colorProps.gray700);
    const inputBoxShadow = get(props,"inputBG",`inset 0 1px 1px ${rgba(colorProps.black,0.075)}`);
    const inputBtnBorderWidth = buttonProps.inputBorderWidth;
    const inputHeightInner = get(props,"inputHeightInner",add(multiply(fontProps.sizeBase,buttonProps.inputLineHeight),multiply(buttonProps.inputPaddingY,2)));
    const inputHeightBorder = get(props,"inputHeightBorder",multiply(inputBtnBorderWidth,2));
    const inputHeight = get(props,"inputHeight",`calc(${inputHeightInner} + ${inputHeightBorder})`);
    const inputHeightInnerSmall = get(props,"inputHeightInnerSmall",add(
      multiply(fontProps.sizeSmall,buttonProps.inputLineHeightSmall),
      multiply(buttonProps.inputPaddingYSmall,2)
    ));
    const inputHeightInnerLarge = get(props,"inputHeightInnerLarge",add(
      multiply(fontProps.sizeLarge,buttonProps.inputLineHeightLarge),
      multiply(buttonProps.inputPaddingYLarge,2)
    ));
    const inputHeightSmall = get(props,"inputHeightSmall",`calc(${inputHeightInnerSmall} + ${inputHeightBorder})`);
    const inputBorderColor = get(props,"inputBorderColor",colorProps.gray400);
    const inputBorderRadius = get(props,"inputBorderRadius",componentProps.borderRadius);
    const inputGroupAddonBG = get(props,"inputGroupAddonBG",colorProps.gray200);
    const customControlIndicatorCheckedColor = get(props,"customControlIndicatorCheckedColor",colorProps.white);
    const customCheckboxIndicatorIndeterminateColor = get(props,"customCheckboxIndicatorIndeterminateColor",customControlIndicatorCheckedColor);
    const customSelectIndicatorColor = get(props,"customSelectIndicatorColor","#333");
    const customSelectFocusBorderColor = get(props,"customSelectFocusBorderColor",lighten(colorProps.themeColors.primary, 0.25));
    const customFileColor = get(props,"customFileColor",inputColor);

    return fuse({
      inputBG,
      inputDisabledBG: colorProps.gray200,
      inputColor,
      inputBorderColor,
      inputBtnBorderWidth,
      inputBoxShadow,

      inputBorderRadius,
      inputBorderRadiusLarge: componentProps.borderRadiusLarge,
      inputBorderRadiusSmall: componentProps.borderRadiusSmall,

      inputFocusBG: inputBG,
      inputFocusBorderColor: lighten(colorProps.themeColors.primary,0.25),
      inputFocusBoxShadow: `${inputBoxShadow}, ${buttonProps.focusBoxShadow}`,
      inputFocusColor: inputColor,

      inputPlaceholderColor: colorProps.gray600,

      inputHeightBorder,

      inputHeightInner,
      inputHeight,

      inputHeightInnerSmall,
      inputHeightSmall,

      inputHeightInnerLarge,
      inputHeightLarge: `calc(${inputHeightInnerLarge} + ${inputHeightBorder})`,

      inputTransition: "border-color ease-in-out .15s, box-shadow ease-in-out .15s",

      textMarginTop: ".25rem",
      checkMarginBottom: ".5rem",
      checkInputGutter: "1.25rem",
      checkInputMarginY: ".25rem",
      checkInputMarginX: ".25rem",

      checkInlineMarginX: ".75rem",

      groupMarginBottom: "1rem",

      inputGroupAddonColor: inputColor,
      inputGroupAddonBG,
      inputGroupAddonBorderColor: inputBorderColor,

      customControlGutter: "1.5rem",
      customControlSpacerY: ".25rem",
      customControlSpacerX: "1rem",

      customControlIndicatorSize: "1rem",
      customControlIndicatorBG: "#ddd",
      customControlIndicatorBgSize: "50% 50%",
      customControlIndicatorBoxShadow: `inset 0 .25rem .25rem ${rgba(colorProps.black,0.1)}`,

      customControlIndicatorDisabledBG: colorProps.gray200,
      customControlDescriptionDisabledColor: colorProps.gray600,

      customControlIndicatorCheckedColor,
      customControlIndicatorCheckedBG: colorProps.themeColors.primary,
      customControlIndicatorCheckedBoxShadow: "none",

      customControlIndicatorFocusBoxShadow: `0 0 0 1px ${bodyProps.bg}, 0 0 0 3px ${colorProps.themeColors.primary}`,

      customControlIndicatorActiveColor: colorProps.white,
      customControlIndicatorActiveBG: lighten(colorProps.themeColors.primary, "35%"),
      customControlIndicatorActiveBoxShadow: "none",

      customCheckboxIndicatorBorderRadius: componentProps.borderRadius,
      customCheckboxIndicatorIconChecked: `url("data:image/svg+xml;charset=utf8,%3Csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 8 8'%3E%3Cpath fill='${customControlIndicatorCheckedColor.replace("#", "%23")}' d='M6.564.75l-3.59 3.612-1.538-1.55L0 4.26 2.974 7.25 8 2.193z'/%3E%3C/svg%3E")`,

      customCheckboxIndicatorIndeterminateBG: colorProps.themeColors.primary,
      customCheckboxIndicatorIndeterminateColor,
      customCheckboxIndicatorIconIndeterminate: `url("data:image/svg+xml;charset=utf8,%3Csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 4 4'%3E%3Cpath stroke='${customCheckboxIndicatorIndeterminateColor.replace("#","%23")}' d='M0 2h4'/%3E%3C/svg%3E")`,
      customCheckboxIndicatorIndeterminateBoxShadow: "none",

      customRadioIndicatorBorderRadius: "50%",
      customRadioIndicatorIconChecked: `url("data:image/svg+xml;charset=utf8,%3Csvg xmlns='http://www.w3.org/2000/svg' viewBox='-4 -4 8 8'%3E%3Ccircle r='3' fill='${customControlIndicatorCheckedColor.replace("#","%23")}'/%3E%3C/svg%3E")`,

      customSelectPaddingY: ".375rem",
      customSelectPaddingX: ".75rem",
      customSelectHeight: inputHeight,
      customSelectIndicatorPadding: "1rem", // Extra padding to account for the presence of the background-image based indicator
      customSelectLineHeight: buttonProps.inputLineHeight,
      customSelectColor: inputColor,
      customSelectDisabledColor: colorProps.gray600,
      customSelectBG: colorProps.white,
      customSelectDisabledBG: colorProps.gray200,
      customSelectBgSize: "8px 10px",
      customSelectIndicatorColor,
      customSelectIndicator: `url("data:image/svg+xml;charset=utf8,%3Csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 4 5'%3E%3Cpath fill='${customSelectIndicatorColor.replace("#", "%23")}' d='M2 0L0 2h4zm0 5L0 3h4z'/%3E%3C/svg%3E")`,
      customSelectBorderWidth: buttonProps.inputBorderWidth,
      customSelectBorderColor: inputBorderColor,
      customSelectBorderRadius: componentProps.borderRadius,

      customSelectFocusBorderColor,
      customSelectFocusBoxShadow: `inset 0 1px 2px ${rgba(colorProps.black, 0.075)}, 0 0 5px ${rgba(customSelectFocusBorderColor, 0.5)}`,

      customSelectFontSizeSmall: "75%",
      customSelectHeightSm: inputHeightSmall,

      customFileHeight: inputHeight,
      customFileWidth: "14rem",
      customFileFocusBoxShadow: `0 0 0 .075rem ${colorProps.white}, 0 0 0 .2rem ${colorProps.themeColors.primary}`,

      customFilePaddingY: buttonProps.inputPaddingY,
      customFilePaddingX: buttonProps.inputPaddingX,
      customFileLineHeight: buttonProps.inputLineHeight,
      customFileColor,
      customFileBG: inputBG,
      customFileBorderWidth: buttonProps.inputBorderWidth,
      customFileBorderColor: inputBorderColor,
      customFileBorderRadius: inputBorderRadius,
      customFileBoxShadow: inputBoxShadow,
      customFileButtonColor: customFileColor,
      customFileButtonBG: inputGroupAddonBG,
      customFileText: {
        placeholder: {
          en: "Choose file...",
        },
        buttonLabel: {
          en: "Browse",
        }
      },
    },props);
  },
};



// Form validation
export const formFeedback = {
  normalize(props,{getProps}) {
    const {colorProps} = getProps({color});
    return fuse({
      validColor: colorProps.themeColors.success,
      invalidColor: colorProps.themeColors.danger,
    },props);
  },
};



// Dropdowns
//
// Dropdown menu container and contents.

// $dropdown-min-width:             10rem !default;
// $dropdown-padding-y:             .5rem !default;
// $dropdown-spacer:                .125rem !default;
// $dropdown-bg:                    $white !default;
// $dropdown-border-color:          rgba($black,.15) !default;
// $dropdown-border-width:          $border-width !default;
// $dropdown-divider-bg:            $gray-200 !default;
// $dropdown-box-shadow:            0 .5rem 1rem rgba($black,.175) !default;
//
// $dropdown-link-color:            $gray-900 !default;
// $dropdown-link-hover-color:      darken($gray-900, 5%) !default;
// $dropdown-link-hover-bg:         $gray-100 !default;
//
// $dropdown-link-active-color:     $component-active-color !default;
// $dropdown-link-active-bg:        $component-active-bg !default;
//
// $dropdown-link-disabled-color:   $gray-600 !default;
//
// $dropdown-item-padding-y:        .25rem !default;
// $dropdown-item-padding-x:        1.5rem !default;
//
// $dropdown-header-color:          $gray-600 !default;


// Z-index master list
//
// Warning: Avoid customizing these values. They're used for a bird's eye view
// of components dependent on the z-axis and are designed to all work together.

export const zindex = {
  defaults: {
    dropdown: 1000,
    sticky: 1020,
    fixed: 1030,
    modalBackdrop: 1040,
    modal: 1050,
    popover: 1060,
    tooltip: 1070,
  }
};

// Navs

export const nav = {

  normalize(props,{getProps}) {

    const {componentProps,colorProps,bodyProps} = getProps({component,color,body});

    return fuse({

      linkPaddingY: ".5rem",
      linkPaddingX: "1rem",
      linkDisabledColor: colorProps.gray600,

      tabsBorderColor: "#ddd",
      tabsBorderWidth: componentProps.borderWidth,
      tabsBorderRadius: componentProps.borderEadius,
      tabsLinkHoverBorderColor: colorProps.gray200,
      tabsLinkActiveColor: colorProps.gray700,
      tabsLinkActiveBG: bodyProps.bg,
      tabsLinkActiveBorderColor: "#ddd",

      pillsBorderRadius: componentProps.borderRadius,
      pillsLinkActiveColor: componentProps.activeColor,
      pillsLinkActiveBG: componentProps.activeBG,
    },props);
  }
};

//
// // Navbar
//
// $navbar-padding-y:                  ($spacer / 2) !default;
// $navbar-padding-x:                  $spacer !default;
//
// $navbar-brand-font-size:            $font-size-lg !default;
// // Compute the navbar-brand padding-y so the navbar-brand will have the same height as navbar-text and nav-link
// $nav-link-height:                   $navbar-brand-font-size * $line-height-base !default;
// $navbar-brand-height:               ($font-size-base * $line-height-base + $nav-link-padding-y * 2) !default;
// $navbar-brand-padding-y:            ($navbar-brand-height - $nav-link-height) / 2 !default;
//
// $navbar-toggler-padding-y:           .25rem !default;
// $navbar-toggler-padding-x:           .75rem !default;
// $navbar-toggler-font-size:           $font-size-lg !default;
// $navbar-toggler-border-radius:       $btn-border-radius !default;
//
// $navbar-dark-color:                 rgba($white,.5) !default;
// $navbar-dark-hover-color:           rgba($white,.75) !default;
// $navbar-dark-active-color:          rgba($white,1) !default;
// $navbar-dark-disabled-color:        rgba($white,.25) !default;
// $navbar-dark-toggler-icon-bg: str-replace(url("data:image/svg+xml;charset=utf8,%3Csvg viewBox='0 0 30 30' xmlns='http://www.w3.org/2000/svg'%3E%3Cpath stroke='#{$navbar-dark-color}' stroke-width='2' stroke-linecap='round' stroke-miterlimit='10' d='M4 7h22M4 15h22M4 23h22'/%3E%3C/svg%3E"), "#", "%23") !default;
// $navbar-dark-toggler-border-color:  rgba($white,.1) !default;
//
// $navbar-light-color:                rgba($black,.5) !default;
// $navbar-light-hover-color:          rgba($black,.7) !default;
// $navbar-light-active-color:         rgba($black,.9) !default;
// $navbar-light-disabled-color:       rgba($black,.3) !default;
// $navbar-light-toggler-icon-bg: str-replace(url("data:image/svg+xml;charset=utf8,%3Csvg viewBox='0 0 30 30' xmlns='http://www.w3.org/2000/svg'%3E%3Cpath stroke='#{$navbar-light-color}' stroke-width='2' stroke-linecap='round' stroke-miterlimit='10' d='M4 7h22M4 15h22M4 23h22'/%3E%3C/svg%3E"), "#", "%23") !default;
// $navbar-light-toggler-border-color: rgba($black,.1) !default;



// Pagination

// $pagination-padding-y:                .5rem !default;
// $pagination-padding-x:                .75rem !default;
// $pagination-padding-y-sm:             .25rem !default;
// $pagination-padding-x-sm:             .5rem !default;
// $pagination-padding-y-lg:             .75rem !default;
// $pagination-padding-x-lg:             1.5rem !default;
// $pagination-line-height:              1.25 !default;
//
// $pagination-color:                     $link-color !default;
// $pagination-bg:                        $white !default;
// $pagination-border-width:              $border-width !default;
// $pagination-border-color:              #ddd !default;
//
// $pagination-hover-color:               $link-hover-color !default;
// $pagination-hover-bg:                  $gray-200 !default;
// $pagination-hover-border-color:        #ddd !default;
//
// $pagination-active-color:              $white !default;
// $pagination-active-bg:                 theme-color("primary") !default;
// $pagination-active-border-color:       theme-color("primary") !default;
//
// $pagination-disabled-color:            $gray-600 !default;
// $pagination-disabled-bg:               $white !default;
// $pagination-disabled-border-color:     #ddd !default;


// Jumbotron

// $jumbotron-padding:              2rem !default;
// $jumbotron-bg:                   $gray-200 !default;


// Cards

// $card-spacer-y:            .75rem !default;
// $card-spacer-x:            1.25rem !default;
// $card-border-width:        $border-width !default;
// $card-border-radius:       $border-radius !default;
// $card-border-color:        rgba($black,.125) !default;
// $card-inner-border-radius: calc(#{$card-border-radius} - #{$card-border-width}) !default;
// $card-cap-bg:              rgba($black, .03) !default;
// $card-bg:                  $white !default;
//
// $card-img-overlay-padding: 1.25rem !default;
//
// $card-group-margin:         ($grid-gutter-width / 2) !default;
// $card-deck-margin:          $card-group-margin !default;
//
// $card-columns-count:        3 !default;
// $card-columns-gap:          1.25rem !default;
// $card-columns-margin:       $card-spacer-y !default;


// Tooltips

// $tooltip-max-width:           200px !default;
// $tooltip-color:               $white !default;
// $tooltip-bg:                  $black !default;
// $tooltip-opacity:             .9 !default;
// $tooltip-padding-y:           3px !default;
// $tooltip-padding-x:           8px !default;
// $tooltip-margin:              0 !default;
//
//
// $tooltip-arrow-width:         5px !default;
// $tooltip-arrow-height:        5px !default;
// $tooltip-arrow-color:         $tooltip-bg !default;


// Popovers

// $popover-inner-padding:               1px !default;
// $popover-bg:                          $white !default;
// $popover-max-width:                   276px !default;
// $popover-border-width:                $border-width !default;
// $popover-border-color:                rgba($black,.2) !default;
// $popover-box-shadow:                  0 5px 10px rgba($black,.2) !default;
//
// $popover-header-bg:                    darken($popover-bg, 3%) !default;
// $popover-header-color:                 $headings-color !default;
// $popover-header-padding-y:             8px !default;
// $popover-header-padding-x:             14px !default;
//
// $popover-body-color:               $body-color !default;
// $popover-body-padding-y:           9px !default;
// $popover-body-padding-x:           14px !default;
//
// $popover-arrow-width:                 10px !default;
// $popover-arrow-height:                5px !default;
// $popover-arrow-color:                 $popover-bg !default;
//
// $popover-arrow-outer-width:           ($popover-arrow-width + 1px) !default;
// $popover-arrow-outer-color:           fade-in($popover-border-color, .05) !default;


// Badges

// $badge-font-size:             75% !default;
// $badge-font-weight:           $font-weight-bold !default;
// $badge-padding-y:             .25em !default;
// $badge-padding-x:             .4em !default;
// $badge-border-radius:         $border-radius !default;
//
// $badge-pill-padding-x:        .6em !default;
// // Use a higher than normal value to ensure completely rounded edges when
// // customizing padding or font-size on labels.
// $badge-pill-border-radius:    10rem !default;


// Modals

// Padding applied to the modal body
// $modal-inner-padding:         15px !default;
//
// $modal-dialog-margin:         10px !default;
// $modal-dialog-margin-y-sm-up: 30px !default;
//
// $modal-title-line-height:     $line-height-base !default;
//
// $modal-content-bg:               $white !default;
// $modal-content-border-color:     rgba($black,.2) !default;
// $modal-content-border-width:     $border-width !default;
// $modal-content-box-shadow-xs:    0 3px 9px rgba($black,.5) !default;
// $modal-content-box-shadow-sm-up: 0 5px 15px rgba($black,.5) !default;
//
// $modal-backdrop-bg:           $black !default;
// $modal-backdrop-opacity:      .5 !default;
// $modal-header-border-color:   $gray-200 !default;
// $modal-footer-border-color:   $modal-header-border-color !default;
// $modal-header-border-width:   $modal-content-border-width !default;
// $modal-footer-border-width:   $modal-header-border-width !default;
// $modal-header-padding:        15px !default;
//
// $modal-lg:                    800px !default;
// $modal-md:                    500px !default;
// $modal-sm:                    300px !default;
//
// $modal-transition:            transform .3s ease-out !default;


// Alerts
//
// Define alert colors, border radius, and padding.

export const alert = {
  normalize(props,{getProps}) {
    const {componentProps,fontProps} = getProps({component,font});
    return fuse({
      paddingY: ".75rem",
      paddingX: "1.25rem",
      marginBottom: "1rem",
      borderRadius: componentProps.borderRadius,
      linkFontWeight: fontProps.weightBold,
      borderWidth: componentProps.borderWidth,
    },props);
  }
};


// Progress bars

// $progress-height:               1rem !default;
// $progress-font-size:            .75rem !default;
// $progress-bg:                   $gray-200 !default;
// $progress-border-radius:        $border-radius !default;
// $progress-box-shadow:           inset 0 .1rem .1rem rgba($black,.1) !default;
// $progress-bar-color:            $white !default;
// $progress-bar-bg:               theme-color("primary") !default;
// $progress-bar-animation-timing: 1s linear infinite !default;
// $progress-bar-transition:       width .6s ease !default;

// List group

// $list-group-bg:                  $white !default;
// $list-group-border-color:        rgba($black,.125) !default;
// $list-group-border-width:        $border-width !default;
// $list-group-border-radius:       $border-radius !default;
//
// $list-group-item-padding-y:      .75rem !default;
// $list-group-item-padding-x:      1.25rem !default;
//
// $list-group-hover-bg:                 $gray-100 !default;
// $list-group-active-color:             $component-active-color !default;
// $list-group-active-bg:                $component-active-bg !default;
// $list-group-active-border-color:      $list-group-active-bg !default;
//
// $list-group-disabled-color:      $gray-600 !default;
// $list-group-disabled-bg:         $list-group-bg !default;
//
// $list-group-action-color:             $gray-700 !default;
// $list-group-action-hover-color:       $list-group-action-color !default;
//
// $list-group-action-active-color:      $body-color !default;
// $list-group-action-active-bg:         $gray-200 !default;


// Image thumbnails

// $thumbnail-padding:           .25rem !default;
// $thumbnail-bg:                $body-bg !default;
// $thumbnail-border-width:      $border-width !default;
// $thumbnail-border-color:      #ddd !default;
// $thumbnail-border-radius:     $border-radius !default;
// $thumbnail-box-shadow:        0 1px 2px rgba($black,.075) !default;
// $thumbnail-transition:        all .2s ease-in-out !default;


// Figures

// $figure-caption-font-size: 90% !default;
// $figure-caption-color:     $gray-600 !default;


// Breadcrumbs

// $breadcrumb-padding-y:          .75rem !default;
// $breadcrumb-padding-x:          1rem !default;
// $breadcrumb-item-padding:       .5rem !default;
//
// $breadcrumb-margin-bottom:      1rem !default;
//
// $breadcrumb-bg:                 $gray-200 !default;
// $breadcrumb-divider-color:      $gray-600 !default;
// $breadcrumb-active-color:       $gray-600 !default;
// $breadcrumb-divider:            "/" !default;


// Carousel

// $carousel-control-color:                      $white !default;
// $carousel-control-width:                      15% !default;
// $carousel-control-opacity:                    .5 !default;
//
// $carousel-indicator-width:                    30px !default;
// $carousel-indicator-height:                   3px !default;
// $carousel-indicator-spacer:                   3px !default;
// $carousel-indicator-active-bg:                $white !default;
//
// $carousel-caption-width:                      70% !default;
// $carousel-caption-color:                      $white !default;
//
// $carousel-control-icon-width:                 20px !default;
//
// $carousel-control-prev-icon-bg: str-replace(url("data:image/svg+xml;charset=utf8,%3Csvg xmlns='http://www.w3.org/2000/svg' fill='#{$carousel-control-color}' viewBox='0 0 8 8'%3E%3Cpath d='M4 0l-4 4 4 4 1.5-1.5-2.5-2.5 2.5-2.5-1.5-1.5z'/%3E%3C/svg%3E"), "#", "%23") !default;
// $carousel-control-next-icon-bg: str-replace(url("data:image/svg+xml;charset=utf8,%3Csvg xmlns='http://www.w3.org/2000/svg' fill='#{$carousel-control-color}' viewBox='0 0 8 8'%3E%3Cpath d='M1.5 0l-1.5 1.5 2.5 2.5-2.5 2.5 1.5 1.5 4-4-4-4z'/%3E%3C/svg%3E"), "#", "%23") !default;
//
// $carousel-transition:           transform .6s ease !default;


// Close

// $close-font-size:             $font-size-base * 1.5 !default;
// $close-font-weight:           $font-weight-bold !default;
// $close-color:                 $black !default;
// $close-text-shadow:           0 1px 0 $white !default;

// Code

// $code-font-size:              90% !default;
// $code-padding-y:              .2rem !default;
// $code-padding-x:              .4rem !default;
// $code-color:                  #bd4147 !default;
// $code-bg:                     $gray-100 !default;
//
// $kbd-color:                   $white !default;
// $kbd-bg:                      $gray-900 !default;
//
// $pre-color:                   $gray-900 !default;
// $pre-scrollable-max-height:   340px !default;
